<?php

declare(strict_types=1);

namespace mirzaev\ebala\models;

// Project files
use mirzaev\ebala\models\traits\instance,
	mirzaev\ebala\models\traits\status,
	mirzaev\ebala\models\account,
	mirzaev\ebala\models\worker,
	mirzaev\ebala\models\market;

// Библиотека для ArangoDB
use ArangoDBClient\Document as _document;

// Фреймворк ArangoDB
use mirzaev\arangodb\collection,
	mirzaev\arangodb\document;

// Встроенные библиотеки
use exception;

/**
 * Модель сокета
 *
 * Управляет записью и чтением настроек сайта из ArangoDB
 *
 * @package mirzaev\ebala\models
 * @author Arsen Mirzaev Tatyano-Muradovich <arsen@mirzaev.sexy>
 */
final class socket extends core
{
	use instance, status;

	/**
	 * Collection name in ArangoDB
	 */
	final public const COLLECTION = 'socket';

	/**
	 * Инстанция документа в базе данных 
	 */
	protected readonly _document $document;

	/**
	 * Подключение к сессии
	 *
	 * @param string $session Идентификатор сессии
	 * @param string $key Ключ для регистрации (из документа в static::COLLECTION)
	 * @param array &$errors Реестр ошибок
	 *
	 * @return bool Сокет подключен к сессии?
	 */
	public static function session(string $session, string $key, array &$errors = []): bool
	{
		try {
			if (
				collection::init(static::$arangodb->session, self::COLLECTION)
				&& collection::init(static::$arangodb->session, session::COLLECTION)
				&& collection::init(static::$arangodb->session, $edge = session::COLLECTION . '_edge_' . self::COLLECTION, true)
			) {
				// Инициализирована коллекция

				// Чтение сокета
				$socket = self::read(
					filter: "d.key == \"$key\" && d.expires > DATE_NOW() / 1000",
					sort: 'd.created desc, d.expires desc',
					amount: 1,
				);

				if ($socket instanceof _document) {
					// Найден сокет

					if (document::write(static::$arangodb->session, $edge, [
						'_from' => $session,
						'_to' => $socket->getId()
					])) {
						// Записано ребро: СЕССИЯ -> СОКЕТ

						// Возврат (успех)
						return true;
					} else throw new exception('Не удалось записать изменения в базу данных');
				} else throw new exception('Не удалось найти сокет');
			} else throw new exception('Не удалось инициализировать коллекции');
		} catch (exception $e) {
			// Write to the errors registry
			$errors['socket'][] = [
				'text' => $e->getMessage(),
				'file' => $e->getFile(),
				'line' => $e->getLine(),
				'stack' => $e->getTrace()
			];
		}

		// Возврат (провал)
		return false;
	}

	/**
	 * Найти аккаунт
	 *
	 * Ищет аккаунт по идентификатору подключения к сокету
	 *
	 * @param int $socket Идентификатор подключения к сокету (во внутренней базе данных OpenSwoole)
	 * @param array &$errors Реестр ошибок
	 *
	 * @returnaccount|null Аккаунт, если найден
	 */
	public static function account(int $socket, array &$errors = []): ?account
	{
		try {
			if (
				collection::init(static::$arangodb->session, self::COLLECTION)
				&& collection::init(static::$arangodb->session, session::COLLECTION)
				&& collection::init(static::$arangodb->session, account::COLLECTION)
				&& collection::init(static::$arangodb->session, session::COLLECTION . '_edge_' . self::COLLECTION, true)
				&& collection::init(static::$arangodb->session, session::COLLECTION . '_edge_' . account::COLLECTION, true)
			) {
				// Инициализированы коллекции

				// Инициализация инстанции аккаунта
				$account = new account;

				// Поиск инстанции аккаунта в базе данных
				$instance = $account->instance(collection::search(static::$arangodb->session, sprintf(
					<<<AQL
						FOR a IN 1 OUTBOUND 
							(FOR s IN 1 INBOUND 
								(FOR s IN %s
									FILTER s.id == %s && s.expires > DATE_NOW() / 1000 
									SORT s.created desc, s.expires desc
									LIMIT 1 
									RETURN s
								)[0]._id 
								 %s
								RETURN s
							)[0]._id 
							%s 
							RETURN a
					AQL,
					socket::COLLECTION,
					$socket,
					session::COLLECTION . '_edge_' . socket::COLLECTION,
					session::COLLECTION . '_edge_' . account::COLLECTION,
				)));

				// Возврат (успех)
				return $instance instanceof _document ? $account : throw new exception('Не удалось найти инстанцию аккаунта в базе данных');
			} else throw new exception('Не удалось инициализировать коллекцию');
		} catch (exception $e) {
			// Запись в реестр ошибок
			$errors['socket'][] = [
				'text' => $e->getMessage(),
				'file' => $e->getFile(),
				'line' => $e->getLine(),
				'stack' => $e->getTrace()
			];
		}

		// Возврат (провал)
		return null;
	}
}
